@@ -1,7 +1,8 @@
 import itertools
+from typing import List
 
 from django.db import connection
-from django.db.models import Manager
+from django.db.models import Manager, Model
 from django.db.models.query import QuerySet
 from django.dispatch import Signal
 from querybuilder.query import Query
@@ -70,6 +71,18 @@ def _get_prepped_model_field(model_obj, field):
     return value
 
 
+def _fetch_models_by_pk(queryset: QuerySet, models: List[Model]) -> List[Model]:
+    """
+    If the given list of model objects is not empty, return a list of newly fetched models.
+    This is important when dependent consumers need relationships hydrated after bulk creating models
+    """
+    if not models:
+        return models
+    return list(
+        queryset.filter(pk__in=[model.pk for model in models])
+    )
+
+
 def bulk_upsert(
     queryset, model_objs, unique_fields, update_fields=None, return_upserts=False, return_upserts_distinct=False,
     sync=False, native=False
@@ -216,10 +229,10 @@ def bulk_upsert(
     # Optionally return the bulk upserted values
     if return_upserts_distinct:
         # return a list of lists, the first being the updated models, the second being the newly created objects
-        return model_objs_to_update, created_models
+        return model_objs_to_update, _fetch_models_by_pk(queryset, created_models)
     if return_upserts:
         # return a union list of created and updated models
-        return model_objs_to_update + created_models
+        return model_objs_to_update + _fetch_models_by_pk(queryset, created_models)
 
 
 def bulk_upsert2(
